import moment from 'moment';
import {
  clientRpcJsonIism,
  clientRpcJsonIismGetChannel,
  clientRpcJsonIismGetChannels,
  SupportedChannels
} from './Config';

export let channels = []
export let updateChannels = []
export let intervalCheckUpdate = true;
export let intervalCheckUpdateTime = 10000;
export let intervalIsOnlineTime = 1000;
export let intervalCheckUpdateIdCount = 0;
export let checkUpdateDevicesInterval = null;
export let isOnlineInterval = null;
export let countUpdate = 0

export const SetDeviceIntervalUpdate = (data) => {
  intervalCheckUpdate = data
}
export const SetDeviceIntervalUpdateTime = (data) => {
  intervalCheckUpdateTime = data
}
export const SetIsOnlineIntervalTime = (data) => {
  intervalIsOnlineTime = data
}

export const UpdateDevicesInterval = (socket, serial) => {
  if (checkUpdateDevicesInterval !== null) {
    clearInterval(checkUpdateDevicesInterval);
  }
  checkUpdateDevicesInterval = setInterval(async () => {
    await CallUpdateDevices(socket, serial);
  }, intervalCheckUpdateTime)
}
export const IsOnlineInterval = (socket, serial) => {
  if (isOnlineInterval !== null) {
    clearInterval(isOnlineInterval);
  }
  isOnlineInterval = setInterval(async () => {
    socket.emit("ccu/isOnline", { serial })
  }, intervalIsOnlineTime)
}

const setUpdateChannels = () => {
  updateChannels = channels.filter((channel) => {
    return SupportedChannels[channel.channelHssType] !== undefined;
  })
}

export const CallGetChannels = async (socket, serial) => {
  let request = await clientRpcJsonIismGetChannels.call(
    {
      "jsonrpc": "2.0",
      "version": "1.1",
      "method": "Event.subscribe",
      "params": {
        "timestamps": "12.06.2022 18:11:07"
      },
      "id": "\"GetChannels\""
    }
  );
  if (request.error === null && request.result.length > 0) {
    channels = request.result
    setUpdateChannels()
    socket.emit("ccu/setChannels", channels)
    socket.emit("ccu/setUpdateChannels", updateChannels)
  } else if (request.error !== null) {
    socket.emit("ccu/getChannelsError", { serial, request })
  }
}
export const CallUpdateDevice = async (channel, socket, serial) => {
    console.log("CallUpdateDevice singel", channel, serial)
  let update = await clientRpcJsonIismGetChannel.call(
    {
      "jsonrpc": "2.0",
      "version": "1.1",
      "method": "Event.subscribe",
      "params": {
        "channel": channel
      },
      "id": intervalCheckUpdateIdCount
    }
  );
  update.result = [update.result]
  if (update.error === null && update.result.length > 0) {
    countUpdate++
    console.log("ccu/updateChannel", countUpdate)
    console.log(update)
    socket.emit("ccu/updateChannel", { serial, update })
  } else if (update.error !== null) {
    console.log("ccu/updateChannelError")
    console.log(update)
    socket.emit("ccu/updateChannelError", { serial, update })
  }
}
export const CallUpdateDevices = async (socket, serial) => {
    console.log("CallUpdateDevices many", serial, updateChannels.map((channel) => {return channel.name}).join(","))

  if (intervalCheckUpdate) {
    let unix = moment().subtract(intervalCheckUpdateTime / 1000, 'seconds').unix()
    let update = await clientRpcJsonIism.call(
      {
        "jsonrpc": "2.0",
        "version": "1.1",
        "method": "Event.subscribe",
        "params": {
          "timestamp": unix,
          "channels": updateChannels.map((channel) => {return channel.idChannel}).join(",")
        },
        "id": intervalCheckUpdateIdCount
      }
    );
    intervalCheckUpdateIdCount = intervalCheckUpdateIdCount + 1
    if (update.error === null && update.result.length > 0) {
      countUpdate++
      console.log("ccu/updateChannel", countUpdate)
      console.log(update)
      socket.emit("ccu/updateChannel", { serial, update })
    } else if (update.error !== null) {
      console.log("ccu/updateChannelError")
      console.log(update)
      socket.emit("ccu/updateChannelError", { serial, update })
    }
  }
}
