#!/bin/tclsh
package require HomeMatic
cd /www
set central true

# Zentrale oder Konfigtool (/www/ = Zentrale)
if {$env(DOCUMENT_ROOT) != "\/www\/"} then {
  set central false
}

#####################
# Verwendete Module #
#####################
load tclrpc.so

if {$central} then {
  load tclrega.so
}

source [file join $env(DOCUMENT_ROOT) api/eq3/common.tcl]
source [file join $env(DOCUMENT_ROOT) api/eq3/ipc.tcl]
source [file join $env(DOCUMENT_ROOT) api/eq3/json.tcl]
source [file join $env(DOCUMENT_ROOT) api/eq3/jsonrpc.tcl]
source [file join $env(DOCUMENT_ROOT) api/eq3/hmscript.tcl]
source [file join $env(DOCUMENT_ROOT) api/eq3/event.tcl]
source session.tcl

set script {
    string id;
    boolean firstChannel = true;
    string channelId;
    WriteLine("[")
    foreach(channelId, system.GetVar("channels").Split(",")) {
        var channel = dom.GetObject(channelId.ToInteger());
        string sLastTime = "";
        integer iSeconds = 0;
        if( channel.IsTypeOf( OT_CHANNEL ) )
            {
                sLastTime = channel.LastDPActionTime();
                iSeconds = channel.LastDPActionTimeSeconds();
            }
        if( channel.IsTypeOf( OT_DP ) )
            {
                sLastTime = channel.Timestamp();
                iSeconds = channel.TimestampSeconds();
            }
        if( channel.IsTypeOf( OT_PROGRAM ) )
            {
                sLastTime = channel.ProgramLastExecuteTime();
                iSeconds = channel.ProgramLastExecuteTimeSeconds();
            }
        if( iSeconds == 0 )
            {
                sLastTime = "";
            }
        else
            {
                sLastTime = sLastTime.Format("%d.%m.%Y %H:%M:%S");
            }
        if ((sLastTime != "") && ( iSeconds.ToInteger() >= system.GetVar("timestamp").ToInteger() ) ) {
            if (true != firstChannel) { Write(","); } else { firstChannel = false; }
            WriteLine("{");
                Write("\"idChannel\":\"");
                Write(channel.ID());
                WriteLine("\",");
                Write("\"sLastTime\":\"");
                Write(sLastTime);
                WriteLine("\",");
                Write("\"time\":\"");
                Write(iSeconds.ToInteger());
                WriteLine("\",");
                Write("\"address\":\"");
                Write(channel.Address());
                WriteLine("\",");
                Write("\"name\":\"");
                Write(channel.Name());
                WriteLine("\",");
                Write("\"channelHssType\":\"");
                Write(channel.HssType());
                WriteLine("\"");
                if ((channel.HssType() == "CLIMATECONTROL_FLOOR_TRANSCEIVER") || (channel.HssType() == "SHUTTER_CONTACT") || (channel.HssType() == "SWITCH_TRANSMITTER") || (channel.HssType() == "CLIMATE_TRANSCEIVER") || (channel.HssType() == "HEATING_CLIMATECONTROL_TRANSCEIVER")) {
                    var dbTemp = channel.DPByHssDP("ACTUAL_TEMPERATURE");
                    var dbHumi = channel.DPByHssDP("HUMIDITY");
                    Write(",\"ACTUAL_TEMPERATURE\":\"")
                    Write(dbTemp.State())
                    WriteLine("\",")
                    Write("\"HUMIDITY\":\"")
                    Write(dbHumi.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrHUMIDITY\":\"");
                    Write(dbHumi.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeHUMIDITY\":\"");
                    Write(dbHumi.ValueType())
                    WriteLine("\",")
                    Write("\"ValueTypeStrACTUAL_TEMPERATURE\":\"");
                    Write(dbTemp.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeACTUAL_TEMPERATURE\":\"");
                    Write(dbTemp.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "COND_SWITCH_TRANSMITTER_TEMPERATURE")) {
                    var dbTemp = channel.DPByHssDP("ACTUAL_TEMPERATURE");
                    Write(",\"ACTUAL_TEMPERATURE\":\"")
                    Write(dbTemp.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrACTUAL_TEMPERATURE\":\"");
                    Write(dbTemp.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeACTUAL_TEMPERATURE\":\"");
                    Write(dbTemp.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "HEATING_CLIMATECONTROL_TRANSCEIVER")) {
                    var dbTemp = channel.DPByHssDP("SET_POINT_TEMPERATURE");
                    Write(",\"SET_POINT_TEMPERATURE\":\"")
                    Write(dbTemp.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrSET_POINT_TEMPERATURE\":\"");
                    Write(dbTemp.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeSET_POINT_TEMPERATURE\":\"");
                    Write(dbTemp.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "DIMMER_VIRTUAL_RECEIVER") || (channel.HssType() == "CLIMATECONTROL_FLOOR_TRANSCEIVER") || (channel.HssType() == "BLIND_TRANSMITTER") || (channel.HssType() == "SHUTTER_TRANSMITTER")) {
                    var dbLevel = channel.DPByHssDP("LEVEL");
                    Write(",\"LEVEL\":\"")
                    Write(dbLevel.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrLEVEL\":\"");
                    Write(dbLevel.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeSTATE\":\"");
                    Write(dbLevel.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "BRIGHTNESS_TRANSMITTER")) {
                    var dbAVERAGE_ILLUMINATION = channel.DPByHssDP("AVERAGE_ILLUMINATION");
                    var dbCURRENT_ILLUMINATION = channel.DPByHssDP("CURRENT_ILLUMINATION");
                    var dbHIGHEST_ILLUMINATION = channel.DPByHssDP("HIGHEST_ILLUMINATION");
                    var dbLOWEST_ILLUMINATION = channel.DPByHssDP("LOWEST_ILLUMINATION");
                    Write(",\"AVERAGE_ILLUMINATION\":\"")
                    Write(dbAVERAGE_ILLUMINATION.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrAVERAGE_ILLUMINATION\":\"");
                    Write(dbAVERAGE_ILLUMINATION.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeAVERAGE_ILLUMINATION\":\"");
                    Write(dbAVERAGE_ILLUMINATION.ValueType())
                    WriteLine("\",")
                    Write("\"CURRENT_ILLUMINATION\":\"")
                    Write(dbCURRENT_ILLUMINATION.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrCURRENT_ILLUMINATION\":\"");
                    Write(dbCURRENT_ILLUMINATION.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeCURRENT_ILLUMINATION\":\"");
                    Write(dbCURRENT_ILLUMINATION.ValueType())
                    WriteLine("\",")
                    Write("\"HIGHEST_ILLUMINATION\":\"")
                    Write(dbHIGHEST_ILLUMINATION.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrHIGHEST_ILLUMINATION\":\"");
                    Write(dbHIGHEST_ILLUMINATION.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeHIGHEST_ILLUMINATION\":\"");
                    Write(dbHIGHEST_ILLUMINATION.ValueType())
                    WriteLine("\",")
                    Write("\"LOWEST_ILLUMINATION\":\"")
                    Write(dbLOWEST_ILLUMINATION.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrLOWEST_ILLUMINATION\":\"");
                    Write(dbLOWEST_ILLUMINATION.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeLOWEST_ILLUMINATION\":\"");
                    Write(dbLOWEST_ILLUMINATION.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "PRESENCEDETECTOR_TRANSCEIVER")) {
                    var dbCURRENT_ILLUMINATION = channel.DPByHssDP("CURRENT_ILLUMINATION");
                    var dbPRESENCE_DETECTION_ACTIVE = channel.DPByHssDP("PRESENCE_DETECTION_ACTIVE");
                    var dbPRESENCE_DETECTION_STATE = channel.DPByHssDP("PRESENCE_DETECTION_STATE");
                    Write(",\"CURRENT_ILLUMINATION\":\"")
                    Write(dbCURRENT_ILLUMINATION.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrCURRENT_ILLUMINATION\":\"");
                    Write(dbCURRENT_ILLUMINATION.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeCURRENT_ILLUMINATION\":\"");
                    Write(dbCURRENT_ILLUMINATION.ValueType())
                    WriteLine("\"")

                    Write(",\"PRESENCE_DETECTION_ACTIVE\":\"")
                    Write(dbPRESENCE_DETECTION_ACTIVE.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrPRESENCE_DETECTION_ACTIVE\":\"");
                    Write(dbPRESENCE_DETECTION_ACTIVE.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypePRESENCE_DETECTION_ACTIVE\":\"");
                    Write(dbPRESENCE_DETECTION_ACTIVE.ValueType())
                    WriteLine("\",")
                    Write("\"PRESENCE_DETECTION_STATE\":\"")
                    Write(dbPRESENCE_DETECTION_STATE.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrPRESENCE_DETECTION_STATE\":\"");
                    Write(dbPRESENCE_DETECTION_STATE.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypePRESENCE_DETECTION_STATE\":\"");
                    Write(dbPRESENCE_DETECTION_STATE.ValueType())
                    WriteLine("\"")
                }
                if ((channel.HssType() == "SHUTTER_CONTACT") || (channel.HssType() == "SWITCH_VIRTUAL_RECEIVER") || (channel.HssType() == "SWITCH_TRANSMITTER")) {
                    var dbState = channel.DPByHssDP("STATE");
                    Write(",\"STATE\":\"")
                    Write(dbState.State())
                    WriteLine("\",")
                    Write("\"ValueTypeStrSTATE\":\"");
                    Write(dbState.ValueTypeStr())
                    WriteLine("\",")
                    Write("\"ValueTypeSTATE\":\"");
                    Write(dbState.ValueType());
                    WriteLine("\"");
                }
            Write("}");
        }
    }
    Write("]")
}

array set JSONRPC    ""
set JSONRPC(METHOD)  ""
set JSONRPC(PARAMS)  ""
set JSONRPC(ID)     "\"error\""
set JSONRPC(ID_USED) 1
proc jsonrpc_parse_2 {request} {
  global JSONRPC JSONRPC_ERROR

#  set request [string range $request 1 end]
  if { [catch { array set parsedRequest [encoding convertfrom identity [json_parse $request]]} errMsg] } then {
     jsonrpc_error 100 "invalid request ($errMsg)"
  }
  if { [catch { set JSONRPC(ID) $parsedRequest(id)} ] } then {
    set JSONRPC(ID_USED) 0
  }
  if { [catch { set JSONRPC(PARAMS) $parsedRequest(params)} ] } then {
    jsonrpc_error 103 "params not found in request"
  }
}
if {$env(REQUEST_METHOD) == "POST"} then {
  if { [catch {
    jsonrpc_parse_2 [read stdin $env(CONTENT_LENGTH)]
    array set args   $JSONRPC(PARAMS)

    jsonrpc_response [hmscript $script args]
  }] } then {
    jsonrpc_error 200 "internal error:\n$errorInfo"
  }
}

